<?php

/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

namespace Sources\Components\Framework\Applications;

/**
 * A static register of applications
 * TODO Load non-core applications
 *
 * @author Cruz Bishop
 * @since 0.0.0.2
 */
class ApplicationRegister {
    
    /**
     * The applications that are registered
     * 
     * @var array The applications
     * @since 0.0.0.2
     */
    private static $applications = array();
    
    /**
     * Loads the applications
     * 
     * @since 0.0.0.2 
     */
    public static function load() {
        //Reset the applications array
        static::$applications = array();
        
        //TODO Load applications from a database instead of the filesystem
        //TODO Load applications except for Core
        //Set the core application
        static::$applications["Core"] = new \Applications\Core\CoreApplication();
    }
    
    /**
     * Checks to see if an application exists
     * 
     * @param string $name The name to look for
     * @return boolean True if found, otherwise false
     * @since 0.0.0.2
     */
    public static function doesApplicationExist($name) {
        return (array_key_exists($name, static::$applications));
    }
    
    /**
     * Gets an application
     * 
     * @param string $applicationName The name to search for
     * @return Application The application found
     * @throws Exceptions\ApplicationClassNotApplicationException The application class does not extend Application
     * @throws Exceptions\ApplicationNotFoundException The application could not be found
     * @since 0.0.0.2
     */
    public static function getApplication($applicationName = "Core") {
        
        //See if the application exists
        if (static::doesApplicationExist($applicationName)) {
            //It does! Make sure it's an application
            if (static::$applications[$applicationName] instanceof Application) {
                //It is! TODO Make sure it's activated
                return static::$applications[$applicationName];
            } else {
                //It's not an application?
                throw new Exceptions\ApplicationClassNotApplicationException(
                        $applicationName . "'s class (probably) does not extend Application!");
            }
        } else {
            //Can't find the application. Great.
            throw new Exceptions\ApplicationNotFoundException($applicationName . " was not found");
        }
    }
    
}

?>
